from django.conf import settings
from pycdi.core import DEFAULT_CONTAINER
from pycdi.core import INJECT_ARGS
from pycdi.core import INJECT_KWARGS

CDI_NAME = getattr(settings, 'CDI_NAME', 'cdi')


class CDIMiddleware(object):
    def __init__(self, get_response=None):
        self.get_response = get_response
        self.container = DEFAULT_CONTAINER

    def __call__(self, request):
        sub_container = self.container.sub_container(request)
        setattr(request, CDI_NAME, sub_container)
        return self.get_response(request)

    def process_view(self, request, view_func, view_args, view_kwargs):
        sub_container = getattr(request, CDI_NAME)
        for k, v in view_kwargs.items():
            sub_container.register_instance(v, context=k)
        inject_kwargs = getattr(view_func, INJECT_KWARGS, False)
        if not inject_kwargs:
            return None
        inject_args = getattr(view_func, INJECT_ARGS, False)

        if len(inject_args) > 0 or 'request' in inject_kwargs:
            return sub_container.call(view_func, *view_args, **view_kwargs)
        else:
            return sub_container.call(view_func, request, *view_args, **view_kwargs)



# -*- encoding: utf-8 -*-
__author__ = 'ettore'

import os
from setuptools import setup, find_packages

from django_pycdi import __version__


def read(file_name):
    return open(os.path.join(os.path.dirname(__file__), file_name)).read()


requirements = []
str_version = '.'.join(map(str, __version__))

setup(
    name='django-pycdi',
    version=str_version,
    description='DJango with Code Dependency Injection using PyCDI',
    long_description=read('README.rst'),
    url='https://github.com/ettoreleandrotognoli/django-pycdi',
    download_url='https://github.com/ettoreleandrotognoli/django-pycdi/tree/%s/' % str_version,
    license='BSD',
    author=u'Éttore Leandro Tognoli',
    author_email='ettore.leandro.tognoli@gmail.com',
    packages=find_packages(exclude=['tests', 'demo2', 'demo3']),
    include_package_data=True,
    keywords=['django', 'pycdi', 'cdi', 'di', 'code dependency injection', 'dependency injection'],
    classifiers=[
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
        'Development Status :: 2 - Pre-Alpha',
        'Framework :: Django',
        'License :: OSI Approved :: BSD License',
    ],
    install_requires=[
        'pycdi>=0.0.3',
        'django'
    ],
    # tests_require=[],
)
